#!/bin/sh
# Copyright (c) 2000-2015 Synology Inc. All rights reserved.

######################################################################################################################
# script to generate syslog-ng relative configurations,
#
#   - Archive       | $LOGCENTER_TARGET/etc/syslog-ng/patterndb.d/pkg-LogCenter-localarchive.conf  | control by "localarch" option of this script
#   - BSD Server    | $LOGCENTER_TARGET/etc/syslog-ng/patterndb.d/pkg-LogCenter-bsd.conf      | control by "bsd" option of this script
#   - IETF Server   | $LOGCENTER_TARGET/etc/syslog-ng/patterndb.d/pkg-LogCenter-ietf.conf     | control by "ietf" option of this script
#   - Custsom Server| $LOGCENTER_TARGET/etc/syslog-ng/patterndb.d/pkg-LogCenter-custrule.conf | control by "custrule" option of this script
#   - Client        | $LOGCENTER_TARGET/etc/syslog-ng/patterndb.d/pkg-LogCenter-client.conf   | control by "client" option of this script
#
######################################################################################################################
LOGCENTER_TARGET=/var/packages/LogCenter/target
SYSLOGNG_CONFIG_FOLDER=${LOGCENTER_TARGET}/etc/syslog-ng/patterndb.d
SYSLOGNG_CONFIG_COMMON=${SYSLOGNG_CONFIG_FOLDER}/common.conf
SYSLOGNG_CONFIG_ARCHIVE=${SYSLOGNG_CONFIG_FOLDER}/pkg-LogCenter-localarchive.conf
SYSLOGNG_CONFIG_BSD=${SYSLOGNG_CONFIG_FOLDER}/pkg-LogCenter-bsd.conf
SYSLOGNG_CONFIG_IETF=${SYSLOGNG_CONFIG_FOLDER}/pkg-LogCenter-ietf.conf
SYSLOGNG_CONFIG_CUSTRULE=${SYSLOGNG_CONFIG_FOLDER}/pkg-LogCenter-custrule.conf
SYSLOGNG_CONFIG_CLIENT=${SYSLOGNG_CONFIG_FOLDER}/pkg-LogCenter-client.conf
SYSLOGNG_CONFIG_NOTIFICATION=${SYSLOGNG_CONFIG_FOLDER}/syslog-notification.conf

SYSLOGNG_SERVER_SETTING=${LOGCENTER_TARGET}/service/conf/server.conf
SYSLOGNG_CUSTRULE_SETTING=${LOGCENTER_TARGET}/service/conf/custrule.conf
SYSLOGNG_CLIENT_SETTING=${LOGCENTER_TARGET}/service/conf/client.conf
SYSLOGNG_NOTIFY_SETTING=/etc/synosyslog/notify.conf

SYSLOGNG_TEMPLATE_FOLDER=${LOGCENTER_TARGET}/service/db_template
SYSLOGNG_ARCHIVE_TEMPLATE_FOLDER=${SYSLOGNG_TEMPLATE_FOLDER}/archive
SYSLOGNG_SERVER_TEMPLATE_FOLDER=${SYSLOGNG_TEMPLATE_FOLDER}/server
SYSLOGNG_CUSTRULE_TEMPLATE_FOLDER=${SYSLOGNG_TEMPLATE_FOLDER}/custrule
SYSLOGNG_CLIENT_TEMPLATE_FOLDER=${SYSLOGNG_TEMPLATE_FOLDER}/client

SYSLOGNG_CONFIG_NOTIFICATION=${SYSLOGNG_CONFIG_FOLDER}/syslog-notification.conf

SYSLOGNG_SCRIPT_FOLDER=${LOGCENTER_TARGET}/service/scripts
. ${SYSLOGNG_SCRIPT_FOLDER}/syslog_common.sh
. ${SYSLOGNG_SCRIPT_FOLDER}/syslog_client_util.sh
. ${SYSLOGNG_SCRIPT_FOLDER}/syslog_server_util.sh
. ${SYSLOGNG_SCRIPT_FOLDER}/syslog_localarch_util.sh
. ${SYSLOGNG_SCRIPT_FOLDER}/syslog_custrule_util.sh
. ${SYSLOGNG_SCRIPT_FOLDER}/syslog_notification.sh

Usage(){
    echo "Usage: $0 (localarch|bsd|ietf|custrule|client|notification) (confgen|confclr)"
}

case $1 in
    notification)
        case $2 in
            confgen)
                SyslogNotificationConfigGen
            ;;
            confclr)
                SyslogNotificationConfigClr
            ;;
            *)
                Usage
            ;;
        esac
    ;;
    localarch)
        case $2 in
            confgen)
                SyslogLocalarchConfigGen
            ;;
            confclr)
                SyslogLocalarchConfigClr
            ;;
            *)
                Usage
            ;;
        esac
    ;;
    bsd)
        case $2 in
            confgen)
                SyslogBsdConfigGen
            ;;
            confclr)
                SyslogBsdConfigClr
            ;;
            *)
                Usage
            ;;
        esac
    ;;
    ietf)
        case $2 in
            confgen)
                SyslogIetfConfigGen
            ;;
            confclr)
                SyslogIetfConfigClr
            ;;
            *)
                Usage
            ;;
        esac
    ;;
    custrule)
        case $2 in
            confgen)
                SyslogCustruleConfigGen
            ;;
            confclr)
                SyslogCustruleConfigClr
            ;;
            *)
                Usage
            ;;
        esac
    ;;
    client)
	case $2 in
            confgen)
                SyslogClientConfigGen
            ;;
            confclr)
                SyslogClientConfigClr
            ;;
            *)
                Usage
            ;;
        esac
    ;;
    common)
        case $2 in
            confgen)
                SyslogCommonConfigGen
            ;;
            *)
                Usage
            ;;
        esac
    ;;
    *)
        Usage
    ;;
esac

